package Plugins::Rhapsody;

use Data::Dumper;
use LWP::UserAgent;
use POSIX qw(strftime);
use XML::Simple;
use URI::Escape;
use SOAP::Lite maptype => {};
use HTML::Entities;

use strict;

###############################################################################
sub init {
    main::plugin_register("Rhapsody", "Rhapsody Support", "/Rhapsody", \&Plugins::Rhapsody::html);
    main::http_register_handler("/Rhapsody/index", \&Plugins::Rhapsody::index);

    main::sonos_add_hook("SEARCH", \&Plugins::Rhapsody::search);
    main::sonos_add_hook("CONTAINER_RDCPA", \&Plugins::Rhapsody::container);
    main::sonos_add_hook("ITEM_RDCPA", \&Plugins::Rhapsody::item);
    main::sonos_add_hook("META_RDCPA", \&Plugins::Rhapsody::meta);
    main::sonos_add_hook("META_RDCPI", \&Plugins::Rhapsody::meta);

    $Plugins::Rhapsody::userAgent = LWP::UserAgent->new(timeout => 5);

    $Plugins::Rhapsody::cobrandId = SOAP::Data->name("cobrandId")->value(40134)->type("");
    $Plugins::Rhapsody::filterRightsKey = SOAP::Data->name("filterRightsKey")->value(2)->type("");
    $Plugins::Rhapsody::developerKey = SOAP::Data->name("developerKey")->value("9F7E8I1D3I6H2B0I")->type("");
}

###############################################################################
sub quit {
}

###############################################################################
sub container {
    my ($what, $name) = @_;

    my $tree;
    if (substr($name, 0, 16) eq "RDCPA:GLBARTIST:") {
        my $artistId = SOAP::Data->name("artistId")->value(substr($name,16))->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getArtist($artistId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $album (@{$tree->{albums}}) {
            my %data;
            $data{"upnp:class"} = "object.container.album.musicAlbum";
            $data{"parentID"} = $name;
            $data{"dc:title"} = $album->{displayName};
            $data{"id"} = "RDCPA:GLBALBUM:" . $album->{albumId};
            $data{"res"}->{"content"} = "radea:" . $album->{albumId};
            $data{"res"}->{"content"} = "x-rincon-cpcontainer:RDCPA:GLBALBUM:" . $album->{albumId};
            push(@{$main::CONTAINERS{$name}}, \%data);
        }
    } elsif (substr($name, 0, 15) eq "RDCPA:GLBALBUM:") {
        my $albumId = SOAP::Data->name("albumId")->value(substr($name,15))->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getAlbum($albumId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        foreach my $track (@{$tree->{trackMetadatas}}) {
            my %data;
            $data{"upnp:class"} = "object.item.audioItem.musicTrack";
            $data{"parentID"} = $name;
            $data{"dc:title"} = $track->{name};
            $data{"id"} = "RDCPI:GLBTRACK:" . $track->{trackId};
            $data{"res"}->{"content"} = "radea:" . $track->{trackId} . ".wma";
            push(@{$main::CONTAINERS{$name}}, \%data);
        }
    } else {
        main::Log (1, "Unknown container $name\n");
        return undef;
    }

    return undef;
}

###############################################################################
sub item {
    my ($what, $name) = @_;

    my $tree;
    if (substr($name, 0, 15) eq "RDCPA:GLBALBUM:") {
        my $albumId = SOAP::Data->name("albumId")->value(substr($name,15))->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getAlbum($albumId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        $main::ITEMS{$name}->{"upnp:class"} = "object.container.album.musicAlbum";
        $main::ITEMS{$name}->{"parentID"} = "RDCPA:GLBARTIST:" . $tree->{primaryArtistId};
        $main::ITEMS{$name}->{"dc:title"} = $tree->{displayName};
        $main::ITEMS{$name}->{"id"} = $name;
        $main::ITEMS{$name}->{"res"}->{"content"} = "x-rincon-cpcontainer:RDCPA:GLBALBUM:" . $tree->{albumId};
    } elsif (substr($name, 0, 15) eq "RDCPI:GLBTRACK:") {
        my $trackId = SOAP::Data->name("albumId")->value(substr($name,15))->type("");
        $tree = SOAP::Lite
          -> proxy('http://direct.rhapsody.com/metadata/services/RhapsodyDirectMetadata')
          -> uri('urn:kani')
          -> getTrack($trackId, $Plugins::Rhapsody::developerKey, $Plugins::Rhapsody::cobrandId, $Plugins::Rhapsody::filterRightsKey)
          -> result;

        $main::ITEMS{$name}->{"upnp:class"} = "object.item.audioItem.musicTrack";
        $main::ITEMS{$name}->{"parentID"} = "RDCPA:GLBALBUM:" . $tree->{primaryArtistId};
        $main::ITEMS{$name}->{"dc:title"} = $tree->{displayName};
        $main::ITEMS{$name}->{"id"} = $name;
        $main::ITEMS{$name}->{"res"}->{"content"} = "radea:" . $tree->{trackId} . ".wma";

    } else {
        main::Log (1, "Unknown item $name\n");
        return undef;
    }

    return undef;
}
###############################################################################
sub meta {
    my ($what, $name, $entry) = @_;

    my $metadata = '&lt;DIDL-Lite xmlns:dc=&quot;http://purl.org/dc/elements/1.1/&quot; xmlns:upnp=&quot;urn:schemas-upnp-org:metadata-1-0/upnp/&quot; xmlns:r=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot; xmlns=&quot;urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/&quot;&gt;&lt;item id=&quot;' . $entry->{id} . '&quot; parentID=&quot;' . $entry->{parentID} . '&quot; restricted=&quot;true&quot;&gt;&lt;dc:title&gt;'. $entry->{"dc:title"} .  '&lt;/dc:title&gt;&lt;upnp:class&gt;';

    if ($what eq "META_RDCPI") {
        $metadata .= 'object.item.audioItem.musicTrack';
    } else {
        $metadata .= 'object.container.album.musicAlbum';
    }
    $metadata .= '&lt;/upnp:class&gt;&lt;desc id=&quot;cdudn&quot; nameSpace=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot;&gt;' . $main::SERVICES{Rhapsody} . '&lt;/desc&gt;&lt;/item&gt;&lt;/DIDL-Lite&gt;';

    return decode_entities($metadata);
}
###############################################################################
sub search {
    my ($what, $zone, $results, $search, $maxsearch, $searchartist, $searchalbum, $searchsong) = @_;

    return if (!$main::PREFS{Rhapsody}->{enabled});

    my $url = "http://realsearch.real.com/search/?searchtype=RhapKeyword&query=" . $search . "&size=50&availability=Stream";
    my $response = $Plugins::Rhapsody::userAgent->get($url);

    if (!$response->is_success()) {
        main::Log(2, "Rhapsody search failed: ". Dumper($response));
        return undef;
    }
    my $content = $response->content;
    my $tree = XMLin($content, forcearray => ["artist", "album", "track"],
                               keyattr=>{"search-results"   => "searchcode"});

    if ($searchartist) {
        foreach my $artist (@{$tree->{"search-results"}->{artist}->{artists}->{artist}}) {
            last if (scalar @{$results} > $maxsearch);
            my %data;
            $data{MUSIC_ISSONG} = 0;
            $data{MUSIC_NAME} = encode_entities($artist->{"display-name"});
            $data{MUSIC_PATH} = uri_escape("RDCPA:GLBARTIST:Art." . $artist->{"artist-id"});
            if (defined $zone) {
                $data{MUSIC_ARG} = "zone=" . uri_escape($zone) . 
                                      "&amp;mpath=" . $data{MUSIC_PATH};
            }
            push(@{$results}, \%data);
        }
    }

    if ($searchalbum) {
        foreach my $album (@{$tree->{"search-results"}->{album}->{albums}->{album}}) {
            last if (scalar @{$results} > $maxsearch);
            my %data;
            $data{MUSIC_ISSONG} = 0;
            $data{MUSIC_NAME} = encode_entities($album->{"display-name"} . " - " . $album->{"artist-name"});
            $data{MUSIC_PATH} = uri_escape("RDCPA:GLBALBUM:Alb." . $album->{"album-content-id"});
            if (defined $zone) {
                $data{MUSIC_ARG} = "zone=" . uri_escape($zone) . 
                                      "&amp;mpath=" . $data{MUSIC_PATH};
            }
            push(@{$results}, \%data);
        }
    }

    if ($searchsong) {
        foreach my $track (@{$tree->{"search-results"}->{track}->{tracks}->{track}}) {
            last if (scalar @{$results} > $maxsearch);
            my %data;
            $data{MUSIC_ISSONG} = 1;
            $data{MUSIC_NAME} = encode_entities($track->{"display-name"} . " - " . $track->{"album-title"} . " - " . $track->{"artist-name"});
            $data{MUSIC_PATH} = uri_escape("RDCPI:GLBTRACK:Tra." . $track->{"content-id"});
            if (defined $zone) {
                $data{MUSIC_ARG} = "zone=" . uri_escape($zone) . 
                                      "&amp;mpath=" . $data{MUSIC_PATH};
            }
            push(@{$results}, \%data);
        }
    }

    return undef;
}

###############################################################################
sub html {
    my ($c, $r, $diskpath, $template) = @_;

    $template->param("RHAPSODY_CONFIGED" => exists $main::SERVICES{Rhapsody});
    $template->param("RHAPSODY_ENABLED" => "checked") if ($main::PREFS{Rhapsody}->{enabled});

    undef $Plugins::Rhapsody::Err;
}

###############################################################################
sub index {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;

    $main::PREFS{Rhapsody}->{enabled} = defined $qf{enabled};
    main::sonos_prefsdb_save();
    $c->send_redirect(main::http_base_url($r) . "/Rhapsody/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
}

1;
